﻿using UnityEngine;
using System.Collections;

public class Boat : MonoBehaviour {

	private bool _Banked;
	private Rigidbody2D _Body;
	private enum ButtonStates
	{
		Released,
		OnPress,
		Pressed,
		OnRelease
	}
	private bool[] _checkClear;
	private ButtonStates[] _buttonStates;
	private float[] _animDelay;

	public int[] _players;
	public SpriteRenderer _leftRow;
	public Sprite[] _leftSprites;
	public SpriteRenderer _rightRow;
	public Sprite[] _rightSprites;

	// Use this for initialization
	void Start () {
		_Banked = false;
		_Body = GetComponent<Rigidbody2D> ();
		_buttonStates = new ButtonStates[_players.Length];
		_animDelay = new float[_players.Length];
		_checkClear = new bool[_players.Length];
	}

	void Update()
	{
		bool pressed;
		int pressedCount = 0;
		for (int i = 0; i < _players.Length; i++) {
			pressed = Input.GetAxis ("Cross" + _players[i]) != 0.0f;
			switch (_buttonStates [i]) {
			case ButtonStates.Released:
				if (pressed) {
					_buttonStates [i] = ButtonStates.OnPress;
					_checkClear [i] = true;
					pressedCount++;
				}
				break;
			case ButtonStates.OnPress:
				_buttonStates [i] = ButtonStates.Pressed;
				pressedCount++;
				break;
			case ButtonStates.Pressed:
				if (!pressed) {
					_buttonStates [i] = ButtonStates.OnRelease;
					_checkClear [i] = true;
				} else {
					pressedCount++;
				}
				break;
			case ButtonStates.OnRelease:
				_buttonStates [i] = ButtonStates.Released;
				break;
			default:
				_buttonStates [i] = ButtonStates.Released;
				break;
			}
		}
	}
	
	// Update is called once per frame
	void FixedUpdate () {
		if (!_Banked) {
			// Check for left paddle
			if ((_buttonStates [0] == ButtonStates.OnPress) || 
				(_checkClear[0] && (_buttonStates[0] == ButtonStates.Pressed))) {
				_leftRow.sprite = _leftSprites [1];
			} else if ((_buttonStates [0] == ButtonStates.OnRelease) || 
				(_checkClear[0] && (_buttonStates[0] == ButtonStates.Released))) {
				_leftRow.sprite = _leftSprites [2];
				_animDelay[0] = 0.2f;
				Paddle (2.0f, -2.0f);
			} else if (_animDelay[0] > 0.0f) {
				_animDelay[0] -= Time.deltaTime;
				if (_animDelay[0] <= 0.0f) {
					_leftRow.sprite = _leftSprites [0];
				}
			}
			_checkClear [0] = false;
			// Check for right paddle
			if ((_buttonStates [1] == ButtonStates.OnPress) || 
				(_checkClear[1] && (_buttonStates[1] == ButtonStates.Pressed))) {
				_rightRow.sprite = _rightSprites [1];
			} else if ((_buttonStates [1] == ButtonStates.OnRelease) || 
				(_checkClear[1] && (_buttonStates[1] == ButtonStates.Released))) {
				_rightRow.sprite = _rightSprites [2];
				_animDelay[1] = 0.2f;
				Paddle (2.0f, 2.0f);
			} else if ((_animDelay[1] > 0.0f) && (_rightRow.sprite == _rightSprites [2])) {
				_animDelay[1] -= Time.deltaTime;
				if (_animDelay[1] <= 0.0f) {
					_rightRow.sprite = _rightSprites [0];
				}
			}
			_checkClear [1] = false;
		}
	}

	public bool Banked()
	{
		return _Banked;
	}

	void Paddle(float fwd, float trq)
	{
		_Body.AddForce (fwd * transform.right);
		_Body.AddTorque (trq);
	}

	void OnTriggerExit2D(Collider2D other)
	{
		if (!_Banked) {
			_Body.constraints = RigidbodyConstraints2D.FreezeAll;
			_Body.isKinematic = true;
			_Banked = true;
		}
	}
}
