﻿using UnityEngine;
using System.Collections;

public class DancingPenguin : MonoBehaviour {

	private enum ButtonStates
	{
		Released,
		OnPress,
		Pressed,
		OnRelease
	}
	private ButtonStates[] _buttonStates;
	private SpriteRenderer _sprite;
	private Sprite[][] _AllDances;
	private float _danceDelay;

	public GameDDR _gameMaster;
	public int _player;
	public Sprite[] _Dance1;
	public Sprite[] _Dance2;
	public Sprite[] _Dance3;

	// Use this for initialization
	void Start () {
		_buttonStates = new ButtonStates[_gameMaster._Buttons.Length];
		_sprite = GetComponent<SpriteRenderer> ();
		_AllDances = new Sprite[3][];
		_AllDances [0] = _Dance1;
		_AllDances [1] = _Dance2;
		_AllDances [2] = _Dance3;
		_sprite.sprite = _AllDances[0][0];
	}
	
	// Update is called once per frame
	void Update () {
		bool pressed;
		int score;
		int pressedCount = 0;
		for (int i = 0; i < _gameMaster._ButtonNames.Length; i++) {
			pressed = Input.GetAxis (_gameMaster._ButtonNames [i] + _player) != 0.0f;
			switch (_buttonStates [i]) {
			case ButtonStates.Released:
				if (pressed) {
					_buttonStates [i] = ButtonStates.OnPress;
					pressedCount++;
				}
				break;
			case ButtonStates.OnPress:
				_buttonStates [i] = ButtonStates.Pressed;
				pressedCount++;
				break;
			case ButtonStates.Pressed:
				if (!pressed) {
					_buttonStates [i] = ButtonStates.OnRelease;
				} else {
					pressedCount++;
				}
				break;
			case ButtonStates.OnRelease:
				_buttonStates [i] = ButtonStates.Released;
				break;
			default:
				_buttonStates [i] = ButtonStates.Released;
				break;
			}
		}
		for (int i = 0; i < _gameMaster._ButtonNames.Length; i++) {
			// Updated the state, so use that to decide what to do
			if ((pressedCount == 1) && (_buttonStates [i] == ButtonStates.OnPress)) {
				score = _gameMaster.OnButtonPressed (i, _player);
				if (score < 0) {
					/* Missed */
					DisplayScore (_gameMaster._scoreSprites.Length - 1);
				} else {
					/* TODO Show the success word */
					DisplayScore (score);
				}
				// Animate the first sprite
				_sprite.sprite = _AllDances[i][1];
				_danceDelay = 0.5f;
			}
			/*else if ((_sprite.sprite.Equals(_AllDances[i][1])) && (_buttonStates [i] == ButtonStates.OnRelease)) {
				// Release the initial dance
				_sprite.sprite = _AllDances[i][2];
				_danceDelay = 0.5f;
			}*/
			else if (_danceDelay <= 0.0f)
			{
				if (_sprite.sprite.Equals (_AllDances [i] [1])) {
					_sprite.sprite = _AllDances[i][2];
					_danceDelay = 1.0f;
				}
				else if (_sprite.sprite.Equals (_AllDances [i] [2])) {
					_sprite.sprite = _AllDances [i] [0];
				}
			}
			if (_danceDelay > 0.0f) {
				_danceDelay -= Time.deltaTime;
			}
		}
	}

	void DisplayScore(int index)
	{
		SpriteRenderer sprite;
		Vector3 pos = transform.position;
		pos.y += 1.0f;

		sprite = Instantiate (_gameMaster._scoreSprites [index], pos, Quaternion.identity) as SpriteRenderer;
		sprite.sortingOrder = (int)_gameMaster.GetTime ();
		Destroy (sprite.gameObject, 0.15f);
	}
}
