﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class GameDDR : GameBase {

	private SpriteRenderer[] _Beats;
	private int _nextToPress;
	private int _numToPress;
	private float _nextButtonTimer;
	private float _gameTimer;
	private float _completion;
	private Vector3 _progressScale;
	private bool _queenDanced;
	private bool[] _playersDanced;

	public GameObject _BeatBar;
	public Vector2 _BeatBarHeight = new Vector2(0.1f, 0.25f);
	public Sprite[] _Buttons;
	public string[] _ButtonNames;
	public float _gameTime = 30.0f;
	public float[] _scoreThresholds;
	public float[] _scorePoints;
	public SpriteRenderer[] _scoreSprites;
	public Transform _targetArea;
	public Transform _startArea;
	public float _speed = 1.0f;
	public float _rate = 2.0f;
	public int _chance = 3;
	public float[] _speedMultipliers;
	public Transform _progressBarScaler;
	public Text _UITimer;
	public Queen _queen;

	// Use this for initialization
	void Start () {
		// Get the beats
		_Beats = GetComponentsInChildren<SpriteRenderer> ();
		// Hide 'em
		foreach (SpriteRenderer sprite in _Beats) {
			sprite.enabled = false;
		}
		// Place the bar at the correct level
		_BeatBar.transform.Translate(
			(Camera.main.orthographicSize * Camera.main.aspect * 2.0f * _BeatBarHeight.x) - Camera.main.orthographicSize * Camera.main.aspect, 
			(Camera.main.orthographicSize * 2.0f * _BeatBarHeight.y) - Camera.main.orthographicSize, 
			0.0f);
		// Square the score thresholds so that it's easier to calculate
		for (int i = 0; i < _scoreThresholds.Length; i++) {
			_scoreThresholds [i] *= _scoreThresholds [i];
		}
		_nextButtonTimer = _rate;
		_gameTimer = _gameTime;
		_completion = 0.0f;
		_progressScale = Vector3.one;
		_playersDanced = new bool[4];
	}
	
	// Update is called once per frame
	void Update () {
		int curr = 0;
		int index = 0;
		int changeTo = _nextToPress;
		int numAdjust = 0;
		int danceCount;
		Vector3 dir = new Vector3 (-_speedMultipliers[_gameManager.Stage()-1] * _speed * Time.deltaTime, 0.0f, 0.0f);
		/* Update game */
		_gameTimer -= Time.deltaTime;
		_UITimer.text = _gameTimer.ToString ("#00.00");
		if (_gameTimer < 15.0f) {
			_UITimer.color = Color.red;
		}
		if ((_gameTimer <= 0.0f) || (_completion > 100.0f)) {
			_gameManager.OnEndOfGame (_gameTimer > 0.0f);
		}
		/* Check whether to add a new button */
		_nextButtonTimer -= _speedMultipliers[_gameManager.Stage()-1] * Time.deltaTime;
		if (_nextButtonTimer <= 0.0f) {
			_nextButtonTimer = _rate;
			if (Random.Range (0, _chance) != 0) {
				// Add a new button
				AddButton(Random.Range(0, _gameManager.Stage()));
			}
		}
		while (curr < _numToPress) {
			index = (_nextToPress + curr) % _Beats.Length;
			_Beats [index].transform.Translate (dir);
			if (_Beats [index].transform.position.x <= (_targetArea.position.x))
			{
				if (!_queenDanced)
				{
					_queenDanced = true;
					_queen.Dance (int.Parse(_Beats[index].name));
				}
				else if (_Beats [index].transform.position.x < (_targetArea.position.x - 1.0f)) {
					// Expired
					_Beats [index].enabled = false;
					// Start at the next one
					changeTo = (index + 1) % _Beats.Length;
					numAdjust++;
					_queenDanced = false;
					danceCount = 0;
					for (int i = 0; i < _playersDanced.Length; i++) {
						if (_playersDanced [i]) {
							danceCount++;
							_playersDanced [i] = false;
						}
					}
					// Punish for any that didn't dance for the correct button
					_completion = Mathf.Max (0, _completion + ((_playersDanced.Length - danceCount) * _scorePoints[_scorePoints.Length - 1]));
				}
			}
			// Update the next one
			curr++;
		}
		// Update the stats
		_numToPress -= numAdjust;
		_nextToPress = changeTo;
		UpdateProgressBar ();
	}

	public void AddButton(int button)
	{
		int index = (_nextToPress + _numToPress) % _Beats.Length;
		_Beats [index].sprite = _Buttons [button];
		_Beats [index].transform.position = _startArea.position;
		_Beats [index].enabled = true;
		_Beats [index].name = button.ToString ();
		_numToPress++;
	}

	public int OnButtonPressed(int button, int player)
	{
		if (_Buttons [button].Equals (_Beats [_nextToPress].sprite)) {
			float dist = (_targetArea.position - _Beats[_nextToPress].transform.position).sqrMagnitude;
			int scoreThresh = _scoreThresholds.Length;
			for (int i = 0; (scoreThresh == _scoreThresholds.Length) && (i < _scoreThresholds.Length); i++) {
				if (dist <= _scoreThresholds [i]) {
					scoreThresh = i;
					_completion += _scorePoints [i] / (float)_gameManager.Stage();
					_playersDanced [player - 1] = true;
				}
			}
			if (scoreThresh != _scoreThresholds.Length) {
				return scoreThresh;
			}
		}
		// Wrong button (add negative points from final point element)
		_completion = Mathf.Max (0, _completion + _scorePoints[_scorePoints.Length - 1]);
		return -1;
	}

	public float GetTime()
	{
		return _gameTime - _gameTimer;
	}

	public void UpdateProgressBar()
	{
		_progressScale.x = Mathf.Clamp01(_completion/100.0f);
		_progressBarScaler.localScale = _progressScale;
	}
}
