﻿using UnityEngine;
using UnityEngine.SceneManagement;
using System.Collections;

public class GameManager : MonoBehaviour {

	static private GameManager _instance;
	private States _gameState;
	private int _stage;
	private bool[] _played;
	private float _totalTimePlayed;

	static public GameManager Instance
	{
		get
		{
			if (_instance == null) {
				try {
					_instance = GameObject.Find("GameManager").GetComponent<GameManager>();
				}
				catch {
					Debug.LogError ("Failed to find the game manager, going to set up");
					SceneManager.LoadScene (0);
				}
			}
			return _instance;
		}
	}
	public enum States
	{
		setup,
		splash,
		mainMenu,
		gameBreak,
		gameStage,
		gameOver
	}
	public int[] _StageGameIndices;
	public int[] _BreakGameIndices;
	public int _MaxStages = 3;

	// Use this for initialization
	void Start () {
		DontDestroyOnLoad (gameObject);
		// Validate the set up (all the games + setup + main menu + game over)
		/*if (SceneManager.sceneCountInBuildSettings != (_StageGameIndices.Length + _BreakGameIndices.Length + 3))
		{
			Debug.LogError ("There seem to be some scenes missing. Expected " + (_StageGameIndices.Length + _BreakGameIndices.Length + 3)
				+ ", but was " + SceneManager.sceneCountInBuildSettings);
			// Continue anyway
		}*/
		// Create the boolean to keep track of which have been played
		_played = new bool[_StageGameIndices.Length];
	}
	
	// Update is called once per frame
	void Update () {
		switch (_gameState) {
		case States.setup:
			// Use the stage to list the number of splash screens
			_stage = -1;
			// Load up the splash screen
			_gameState = States.splash;
			SceneManager.LoadScene(1);
			break;
		case States.splash:
			if (_stage == 0) {
				LoadMainMenu ();
			}
			break;
		case States.mainMenu:
			if (Input.GetKeyDown (KeyCode.Escape)) {
				Application.Quit ();
			} else if (Input.GetKeyDown (KeyCode.S) || (Input.GetAxis("Cross1") != 0.0f)) {
				StartNewGame ();
			}
			break;
		case States.gameStage:
			if (Input.GetKeyDown(KeyCode.R)) {
				LoadMainMenu ();
			}
			// Count time
			_totalTimePlayed += Time.deltaTime;
			if (Input.GetKeyDown (KeyCode.S)) {
				OnEndOfGame (true);
			}
			break;
		case States.gameBreak:
			if (Input.GetKeyDown(KeyCode.R)) {
				LoadMainMenu ();
			}
			// Count time
			_totalTimePlayed += Time.deltaTime;
			if (Input.GetKeyDown (KeyCode.S)) {
				OnEndOfGame (true);
			}
			break;
		case States.gameOver:
			if (Input.GetAxis("Cross1") != 0.0f) {
				LoadMainMenu ();
			}
			break;
		default:
			Debug.LogError ("Unhandled state: " + _gameState);
			break;
		}
	}

	public void LoadMainMenu()
	{
		_gameState = States.mainMenu;
		SceneManager.LoadScene (2);
	}

	public int Stage()
	{
		return _stage;
	}

	public States State()
	{
		return _gameState;
	}

	public bool StageCompleted()
	{
		bool complete = true;

		foreach (bool played in _played) {
			complete &= played;
		}

		return complete;
	}

	public void OnStageComplete()
	{
		if (_stage < _MaxStages) {
			// Go onto the next stage
			_stage++;
			// Clear that games have been played
			for (int i = 0; i < _played.Length; i++) {
				_played [i] = false;
			}
			// Choose a random break
			_gameState = States.gameBreak;
			LoadNextGame(true);
		} else {
			// Gameover
			GameOver(true);
		}
	}

	public void GameOver(bool success)
	{
		print("gameover");
		_gameState = States.gameOver;
		if (!success) {
			SceneManager.LoadScene (SceneManager.sceneCountInBuildSettings - 1);
		} else {
			SceneManager.LoadScene (SceneManager.sceneCountInBuildSettings - 2);
		}
	}

	public float GetTotalTime()
	{
		return _totalTimePlayed;
	}

	public void StartNewGame()
	{
		// Clear everything
		for (int i = 0; i < _played.Length; i++) {
			_played [i] = false;
		}
		_gameState = States.gameStage;
		_stage = 1;
		OnEndOfGame (true);
		_totalTimePlayed = 0.0f;
	}

	public void OnEndOfGame(bool success)
	{
		if (!success) {
			GameOver (false);
		}
		else if (_gameState == States.gameBreak) {
			_gameState = States.gameStage;
			LoadNextGame (false);
		}
		else if (StageCompleted ()) {
			OnStageComplete ();
		} else {
			LoadNextGame (false);
		}
	}

	public void LoadNextGame(bool isBreak)
	{
		int index = 1;
		int counter = 0;
		if (isBreak) {
			index = _BreakGameIndices [Random.Range (0, _BreakGameIndices.Length)];
			print ("Loading break");
		} else {
			index = Random.Range (0, _StageGameIndices.Length);
			while (_played [index] && (counter < _StageGameIndices.Length)) {
				index = (index + 1) % _StageGameIndices.Length;
				counter++;
			}
			if (counter == _StageGameIndices.Length) {
				Debug.LogError ("Did a full loop, looks like all games were played!");
			} else {
				_played [index] = true;
				// Get the actual game index
				print("Game scene #" + index + " hasn't been played, loading that");
				index = _StageGameIndices [index];
			}
		}
		print ("Loading game scene " + index);
		SceneManager.LoadScene (index);
	}

	public void OnFadeComplete()
	{
		_stage++;
	}
}
