﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class GameMexicanWave : GameBase {

	private MexicanPenguin[] _Penguins;
	private int[] _UsedPenguins;
	private int _totalPenguins;
	private float _gameTimer;
	private float _waveTimer;
	private int _currentPeng;
	private float _completeTimer;

	public GameObject _PenguinGroup;
	public string[] _Buttons;
	public float _waveThreshold = 0.5f;
	public float _timeLimit = 30.0f;
	public ParticleSystem[] _partyCannons;
	public ParticleSystem[] _successParty;
	public Text _UITimer;
	public GameObject[] _ButtonIcons;

	// Use this for initialization
	void Start () {
		int useIndex = 0;
		bool freeIndex = false;
		Vector3 pos = Vector3.zero;
		float xOff;
		int[] rndIndex;
		bool[] numUsed;
		_Penguins = _PenguinGroup.GetComponentsInChildren<MexicanPenguin> ();
		// Disable the amount base on the current game stage;
		_totalPenguins = _gameManager.Stage() * 4;
		if (_totalPenguins > _Penguins.Length) {
			Debug.LogError ("Not enough penguins... Want " + _totalPenguins + " but only " + _Penguins.Length);
			_totalPenguins = _Penguins.Length;
		}
		_UsedPenguins = new int[_totalPenguins];
		rndIndex = new int[_totalPenguins];
		numUsed = new bool[_totalPenguins];
		// Generate a random player order
		for (int i = 0; i < _totalPenguins; i++) {
			do {
				useIndex = Random.Range (0, _totalPenguins);
			} while(numUsed [useIndex]);
			numUsed [useIndex] = true;
			rndIndex [i] = useIndex;
		}
		// Place the penguins evenly (and randomly)
		xOff = (Camera.main.aspect * Camera.main.orthographicSize * 2) / (_totalPenguins + 1.0f);
		pos.x = xOff - (Camera.main.aspect * Camera.main.orthographicSize);
		for (int i = 0; i < _totalPenguins; i++) {
			do {
				useIndex = Random.Range (0, _Penguins.Length);
				freeIndex = true;
				for (int j = 0; freeIndex && (j < i); j++) {
					freeIndex &= _UsedPenguins[j] != useIndex;
				}
			} while (!freeIndex);
			_UsedPenguins [i] = useIndex;
			// Place
			_Penguins[useIndex].transform.position = pos;
			_Penguins [useIndex]._playerNumber = (rndIndex [i] % 4) + 1;
			_Penguins [useIndex]._button = _Buttons[rndIndex [i]/4];
			pos.x += xOff;
		}
		// Hide penguins not being used
		for (int i = 0; i < _Penguins.Length; i++) {
			freeIndex = false;
			for (int j = 0; j < _totalPenguins; j++) {
				freeIndex |= _UsedPenguins [j] == i;
			}
			_Penguins [i].gameObject.SetActive (freeIndex);
		}
		_currentPeng = 0;
		_gameTimer = _timeLimit;
		_UITimer.text = _gameTimer.ToString ("#00.00");
		_UITimer.color = Color.black;

		for (int i = 0; i < _ButtonIcons.Length; i++) {
			_ButtonIcons [i].SetActive (i < _gameManager.Stage ());
		}
	}
	
	// Update is called once per frame
	void Update () {
		if (_completeTimer == 0.0f) {
			_gameTimer -= Time.deltaTime;
			_UITimer.text = _gameTimer.ToString ("#00.00");
			if (_gameTimer < 15.0f) {
				_UITimer.color = Color.red;
			}
			if (_gameTimer <= 0.0f) {
				_gameManager.OnEndOfGame (false);
			}
			// Check whether to wait for player to cheer
			if (_currentPeng > 0) {
				_waveTimer -= Time.deltaTime;
				if (_waveTimer <= 0.0f) {
					_currentPeng = 0;
				}
			}
			/*
			for (int i = 0; i < _totalPenguins; i++) {
				if (i == _currentPeng) {
					_Penguins [_UsedPenguins [i]].GetComponent<SpriteRenderer> ().color = Color.green;
				} else {
					_Penguins [_UsedPenguins [i]].GetComponent<SpriteRenderer> ().color = Color.white;
				}
			}
			*/
			// Check whether the current peng has cheers
			if (_Penguins [_UsedPenguins [_currentPeng]].Jumping ()) {
				// Make sure the next one isn't jumping
				if (_currentPeng == (_totalPenguins - 1) || !_Penguins [_UsedPenguins [_currentPeng + 1]].Jumping ()) {
					// Celebrate
					foreach (ParticleSystem party in _partyCannons) {
						party.transform.position = _Penguins [_UsedPenguins [_currentPeng]].transform.position;
						party.Emit (25);
					}
					// Move to the next penguin
					_currentPeng++;
					_waveTimer = _waveThreshold;
					if (_currentPeng == _totalPenguins) {
						foreach (ParticleSystem party in _successParty) {
							party.Play ();
						}
						_completeTimer = 2.0f;
					}
				}
			}	
		} else {
			_completeTimer -= Time.deltaTime;
			if (_completeTimer <= 0.0f) {
				_gameManager.OnEndOfGame (true);
			}
		}
	}
}
