﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class GameWineWhine : GameBase {

	private int _bestDevice;
	private enum States
	{
		intro,
		record,
		end
	}
	private States _state;
	private float _timer;
	private float[] _samples;
	private float _vibration;
	private bool _broke;

	public AudioSource _audio;
	public float _gameTime = 10.0f;
	public float _cooldown = 0.5f;
	public float _breakLimit = 2.0f;
	public float _threshold = 0.025f;
	public Text _UITimer;

	// When added to an object, draws colored rays from the
	// transform position.
	public int lineCount = 100;
	/*
	public float size = 3.0f;
	public float freq = 1.0f;
	public float density = 1.0f;
	public float length = 1.0f;
	public Color lineColour;
	*/
	public float maxHeight = 1.0f;
	public GameObject[] _glasses;
	public GameObject _brokenGlass;
	public AnimationCurve _VibrationDist;
	/*
	static Material lineMaterial;
	static void CreateLineMaterial ()
	{
		if (!lineMaterial)
		{
			// Unity has a built-in shader that is useful for drawing
			// simple colored things.
			Shader shader = Shader.Find ("Hidden/Internal-Colored");
			lineMaterial = new Material (shader);
			lineMaterial.hideFlags = HideFlags.HideAndDontSave;
			// Turn on alpha blending
			lineMaterial.SetInt ("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.SrcAlpha);
			lineMaterial.SetInt ("_DstBlend", (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);
			// Turn backface culling off
			lineMaterial.SetInt ("_Cull", (int)UnityEngine.Rendering.CullMode.Off);
			// Turn off depth writes
			lineMaterial.SetInt ("_ZWrite", 0);
		}
	}
	*/

	void Start()
	{
		_state = States.intro;
		_timer = 1.0f;
		SetupDevices ();
		_samples = new float[lineCount];
		_vibration = 0.0f;
		_UITimer.text = _gameTime.ToString("#00.00");
	}

	void SetupDevices()
	{
		int min, max;
		if (Microphone.devices.Length == 0) {
			Debug.LogError ("No microphones found");
		} else {
			for (int i = 0; i < Microphone.devices.Length; i++) {
				Microphone.GetDeviceCaps (Microphone.devices [i], out min, out max);
				print ("Found device " + Microphone.devices [i] + " with min freq: " + min + ", max freq: " + max);
			}
			_bestDevice = Microphone.devices.Length - 1;
		}
	}

	void Update()
	{
		Vector3 offset = Vector3.zero;
		float amp = 0.5f * _VibrationDist.Evaluate(_vibration / _breakLimit);
		float freq = 5.4321f * amp * Time.time;
		switch (_state) {
		case States.intro:
			_timer -= Time.deltaTime;
			if (_timer <= 0.0f) {
				_state = States.record;
				_audio.clip = Microphone.Start (Microphone.devices [_bestDevice], true, (int)_gameTime, 44100);
				_audio.PlayDelayed(0.5f);
				_timer = 0.0f;
				_broke = false;
			}
			break;
		case States.record:
			TestSound ();
			if (_timer < _gameTime) {
				_timer += Time.deltaTime;
				_vibration = Mathf.MoveTowards (_vibration, 0.0f, _cooldown * Time.deltaTime);
				// Move the glass around based on the vibration
				_glasses[2].transform.position = _glasses[1].transform.position = _glasses[0].transform.position;
				offset.x = amp * Mathf.PerlinNoise (freq, freq/2.13f);
				offset.y = amp * Mathf.PerlinNoise (freq/2.15f, freq);
				_glasses [1].transform.Translate (offset);
				_glasses [2].transform.Translate (-offset);
				if (_vibration > _breakLimit) {
					_glasses [0].SetActive (false);
					_glasses [1].SetActive (false);
					_glasses [2].SetActive (false);
					_brokenGlass.SetActive (true);
					_broke = true;
					_timer = _gameTime;
				}
				_UITimer.text = (_gameTime - _timer).ToString("#00.00");
			}
			else
			{
				_state = States.end;
				_timer = 0.0f;
			}
			break;
		case States.end:
			_timer += Time.deltaTime;
			if (_timer > 2.0f) {
				Microphone.End (Microphone.devices [_bestDevice]);
				_gameManager.OnEndOfGame (_broke);
			}
			break;
		}
	}

	void FixedUpdate()
	{

	}

	// Will be called after all regular rendering is done
	//public void OnRenderObject ()
	void TestSound()
	{
		//float scaledStep = length / (float)lineCount;
		//float average = 0.0f;
		float iHeight = 0.0f;
		bool bump = false;
		//_audio.GetSpectrumData(_samples, 0, FFTWindow.Rectangular);
		_audio.GetOutputData(_samples, 0);
		//CreateLineMaterial ();
		// Apply the line material
		//lineMaterial.SetPass (0);

		if (!Microphone.IsRecording (Microphone.devices [_bestDevice])) {
			return;
		}
		/*
		GL.PushMatrix ();
		// Set transformation matrix for drawing to
		// match our transform
		GL.MultMatrix (transform.localToWorldMatrix);
		// Draw lines
		GL.Begin (GL.LINES);
		GL.Color (lineColour);
		*/
		for (int i = 0; i <= (_samples.Length - 1); ++i)
		{
			// One vertex at transform position
			iHeight = _samples[i];
			/*
			GL.Vertex3 ((i * scaledStep) - (length / 2.0f), maxHeight * iHeight, 10);
			GL.Vertex3 (((i + 1) * scaledStep) - (length / 2.0f), maxHeight * _samples[i + 1], 10);
			average += iHeight * maxHeight;
			*/
			bump |= (iHeight * maxHeight) > _threshold;
		}
		/*
		GL.End ();
		GL.PopMatrix ();
		average /= lineCount;
		*/
		//if (bump || (Random.Range(0,3) == 0)) {
			_vibration += Time.deltaTime;
		//}
	}
}
